function [ Settings, Data ] = Read_RFe_V2c_no_laser_probe( folderName, begin_frame, nr_frames )

%   Input
%   fileName    : name of the <*.rfe> file (if 0 file dialog will open)
%   begin_frame : Begin frame (1 to max)
%   nr_frames   : Number of frames to read, 0 for reading all frames
%
%   Output
%   Settings.finfo       : File info
%   Settings.acq         : Settings.acquisition Settings
%   Settings.tran        : Settings.transmission Settings
%   Settings.lm          : Label matrix (16,frame_nr)
%   Data         : RF-data matrix (nr_sampels, line_nr);
%
%
%   Peter Brands, Yannick van Bavel
%

%
%   Get file name if there is no file input available
%
if folderName == 0
    [ fileName, filePath ] = uigetfile( '*.rfe', 'Open RFE data *.rfe file' );
    if fileName == 0 ,return ,end
elseif ~isempty(strfind(folderName, '.rfe'))
    splittedFileName = strsplit(folderName, '\');
    fileName = splittedFileName{end};
    splittedFileName = strsplit(folderName, fileName);
    filePath = splittedFileName{1};
else
    [ fileName, filePath ] = uigetfile( '*.rfe', 'Open RFE data *.rfe file', folderName);
    if fileName == 0 ,return ,end
end
splittedFileName = strsplit(fileName, '.rfe');
Settings.fileName = splittedFileName{1};
Settings.filePath = filePath;
fprintf('Reading data...........');
%
%   Open *.zre file
%
fid = fopen( [filePath fileName], 'r' );

%
%   Start interpertation of file
%
if ( fid == -1 )
   error( 1, 'File "%s" does NOT exist in this directory!\n\n', [filePath fileName] );
else
    
%
%   Read file header
%
    Settings.finfo.id   = fread( fid, 10, 'uint8=>char' );   % File ID "planewave"
    Settings.finfo.ver  = fread( fid, 1,  'uint32'      );   % File version
    Settings.finfo.size = fread( fid, 1,  'uint32'      );   % File size in bytes
    Settings.finfo.date = fread( fid, 20, 'uint8=>char' );   % File date and time stamp
    Settings.finfo.nrf  = fread( fid, 1,  'uint32'      );   % Number of frames
    Settings.finfo.nrl  = fread( fid, 1,  'uint32'      );   % Number of lines
    Settings.finfo.nrs  = fread( fid, 1,  'uint32'      );   % Number of samples
    Settings.finfo.lmp  = fread( fid, 1,  'uint32'      );   % Label matrix offset [bytes]
    Settings.finfo.rfp  = fread( fid, 1,  'uint32'      );   % RF-matrix offset [bytes]

    if begin_frame >= Settings.finfo.nrf
        begin_frame = 0;
    end
    
    if nr_frames == 0
        nr_frames = Settings.finfo.nrf;
    end
    
    if begin_frame + nr_frames > Settings.finfo.nrf
        nr_frames = Settings.finfo.nrf - begin_frame;
    end
    
    Settings.begin_frame = begin_frame;
    Settings.nr_frames = nr_frames;
%
%   Read Settings.acqusition Settings
%
    Settings.acq.size    = fread( fid, 1, 'uint32' );        % Size of the Settings.acqusition Settings struct [bytes]
    Settings.acq.ftime   = fread( fid, 1, 'uint32' );        % Frame time [us]
    Settings.acq.fs      = fread( fid, 1, 'uint32' );        % Sample frequency [Hz]
    Settings.acq.dt      = fread( fid, 1, 'uint32' );        % Settings.acqusition delay [RF sampels]

%
%    Read Settings.transmission Settings
%
    Settings.tran.size               = fread( fid, 1, 'uint32' );
    Settings.tran.halfWaveLength     = fread( fid, 1, 'uint32' );
    Settings.tran.pulseDuration      = fread( fid, 1, 'uint32' );
    Settings.tran.numberHalfWaves    = fread( fid, 1, 'uint32' );
    Settings.tran.phase              = fread( fid, 1, 'uint32' );
    Settings.tran.pulseAmplitude     = fread( fid, 1, 'uint32' );
    
%
%   Read label matrix
%
    Settings.lm = reshape( fread( fid, 16*Settings.finfo.nrf, 'int16' ), 16, Settings.finfo.nrf );
    Settings.lm = Settings.lm(:, begin_frame+1:nr_frames+begin_frame);
%
%   Skip number of frames
%
    fs = Settings.finfo.nrl*Settings.finfo.nrs*2*begin_frame;   % Pointer to RF-frame in bytes
    fseek( fid, fs, 0 );

%
%   Read RF-matrix
%
    dataRead = fread( fid, nr_frames*Settings.finfo.nrs * Settings.finfo.nrl, 'int16' );
    leftFrame = Settings.finfo.nrl*Settings.finfo.nrs*2*(Settings.finfo.nrf-nr_frames-begin_frame);
    fseek( fid, leftFrame, 0 );
    checkRead = fread(fid, 1, 'int16');
    if ~isempty(checkRead)
        error('Seems wrong in reading data!');
    end
    Data.fullFrame = zeros(nr_frames, Settings.finfo.nrs, Settings.finfo.nrl);
    Data.halfFrame = zeros(nr_frames, ceil(Settings.finfo.nrs/2), Settings.finfo.nrl); 
    frameSize = Settings.finfo.nrs * Settings.finfo.nrl;
    tic
    for iFrame = 1:nr_frames
        curFrame = dataRead((iFrame-1)*frameSize+1:iFrame*frameSize);
        curFrame = reshape(curFrame, Settings.finfo.nrs, Settings.finfo.nrl);   
        curHalfFrame = curFrame(1:ceil(Settings.finfo.nrs/2),: );
        curHalfFrame(end-5:end,:) = 0;
        curHalfFrame(1:end-6, :) = detrend(curHalfFrame(1:end-6, :), 'constant');
        Data.fullFrame(iFrame, :, :) =  curFrame;
        Data.halfFrame(iFrame, :, :) =  curHalfFrame;
    end
    toc    
    Settings.fullFrameLength = size(Data.fullFrame, 2);
    Settings.halfFrameLength = size(Data.halfFrame, 2);
% 
%   Close the file
%      
    fclose( fid );
    
end
fprintf('done!\n');


%
%   END
%