% -------------------------------------------------------------------------
% This script is to process the xz calibration of a US probe (no integrated
% laser). The data file name is supposed to be *_*_z_*_x_*.rfe. If you save
% data in another name format, change the part reading x and z positions in
% the "sort files" section.
% The script win run through very data file. In each file, it will
% automatically find the absorber. Some important parameters in the 
% "find roi window" section are:
%    nFitSmplsL, nFitSmplsR: the script will perform a gaussian fit with
%    data from the maximum pixel -nFitSmplsL to +nFitSmplsR
%    xPlane: the x position [1,N] that the absorber is in the imaging plane
% at positions that the image is very noisy, the script might find a wrong
% max pixel, keep an eye on this.
% -------------------------------------------------------------------------
close all
dataPath = 'P:\TNW\BMPI\Users\Ho Nhu Y Nguyen\Published papers\3D view of out-of-plane artifacts\data\xz calibration';
loadProcessedData = true;

%% load processed data
if loadProcessedData
    load([dataPath '\processed data.mat']);
    nZs = length(probeZs);
    nXs = length(probeXs);
else
%% select files
    [fn, fp] = uigetfile('*.rfe','Select the displacing probe PA DATA FILE(s)',...
        'MultiSelect','on', dataPath);
    if length(fn) < 3
        warning('not enough files, at least 3 files');
        return
    end
    nFiles = length(fn);
    fn = fn';

    %% sort files
    times = zeros(nFiles, 1);
    probeXs = zeros(nFiles, 1);
    probeZs = zeros(nFiles, 1);
    for iFile = 1:nFiles
        fileNameInfo = strsplit(fn{iFile}, '_');
        probeXs(iFile) = str2double(fileNameInfo{5});
        probeZs(iFile) = str2double(fileNameInfo{3});
        timeStr = strsplit(fileNameInfo{8}, '-');
        times(iFile) = str2double(timeStr{1})*3600 + str2double(timeStr{2})*60 + str2double(timeStr{3});
    end
    [times, sortOrd] = sort(times);
    fn = fn(sortOrd);
    probeXs = probeXs(sortOrd);
    probeZs = probeZs(sortOrd);
    nZs = sum(probeXs == probeXs(1));
    nXs = nFiles/nZs;
    probeXs = probeXs(1:nZs:end);
    probeZs = probeZs(1:nZs);
    fn = reshape(fn, nZs, nXs);

    %% reconstruct images
    [~,Settings] = read_process_noLaser([fp fn{1,1}],0,-1); % read Settings
    zMinSample = Settings.halfFrameLength;
    rekon = zeros(nZs, nXs, zMinSample, 64); % array containing reconstructed imges
    % [b,a] = butter(5, 5.5e6/(Settings.acq.fs/2), 'low');

    fprintf('Reconstructing images...');
    for iZ=1:nZs
        for iX=1:nXs
            [~, Data] = Read_RFe_V2c_no_laser_probe([fp fn{iZ,iX}], 0, 0);
            curSignal = squeeze(mean(Data.halfFrame));
    %         curSignal = filter(b, a, curSignal);
            tempRekon = rekon_OA_freqdom(curSignal, Settings);
            rekon(iZ,iX,:,:) = tempRekon(end-zMinSample+1:end,:);
        end    
    end
    fprintf('done!\n');
end

%% find roi window
nFitSmplsL = 8;
nFitSmplsR = 8;
xPlane = 17; % x position of the imaging plane
iniPosImgs = squeeze(rekon(1,xPlane,:,:)); % 1: initial z postiion
[~, idx] = max(iniPosImgs(:));
[row, col] = ind2sub(size(iniPosImgs), idx);
z0 = row*Settings.sampleDepth;
x0 = abs(max(abs(probeXs(:) - probeXs(xPlane)))-2.5);
maxDeltaZ = abs(probeZs(end) - probeZs(1));
maxShift = -sqrt(x0^2+(z0+maxDeltaZ)^2)+sqrt(x0^2+z0^2)+maxDeltaZ;
maxSampleShift = ceil(maxShift/Settings.sampleDepth)+nFitSmplsR;
if maxSampleShift>row
    maxSampleShift = row - 5;
end
probeXsRltv = abs(probeXs(:)-probeXs(1));
probeZsRltv = abs(probeZs(:)-probeZs(1))+z0;

%% calculate max Amp
if ~loadProcessedData
    maxAmp = zeros(nZs, nXs); % array containing max recorded Amp at each probe position
    maxPixelXs = zeros(nZs, nXs);
    maxPixelZs = zeros(nZs, nXs);
end
for iZ=1:nZs
    for iX = 1:nXs
        tempRekon = squeeze(rekon(iZ,iX,:,:));
        roi = tempRekon(row-nFitSmplsL:row+maxSampleShift,col-2:col+2);
        [~, linIdx] = max(roi(:));
        [iRow, iCol] = ind2sub(size(roi), linIdx);
        iRow = iRow + row-nFitSmplsL - 1;
        iCol = iCol + col-2 - 1;
        tempRekon(iRow,1:iCol-4) = 0;
        maxPixelZs(iZ,iX) = iRow;
%         if iX==18 && iZ==29
%             tempRekon(iRow, iCol-nFitSmplsL:iCol-3) = 0;
%         end
        tempGaus = fit((iCol-nFitSmplsL:iCol+nFitSmplsR)'*Settings.pitch, tempRekon(iRow,iCol-nFitSmplsL:iCol+nFitSmplsR)', 'gauss1');
        maxAmp(iZ,iX) = tempGaus.a1;
        if iX==1 && iZ==1
            subplot(2,2,[1 3]);
            sp1 = imagesc(Settings.acqY, Settings.acqDepth, tempRekon);
            xlabel('y [mm]');
            ylabel('z [mm]');
            colormap hot
            axis image
            l1 = line([1 64]*Settings.pitch, [iRow iRow]*Settings.sampleDepth, 'Color', 'white');
            
            subplot(2,2,2);
            sp21 = plot((iCol-nFitSmplsL:iCol+nFitSmplsR)*Settings.pitch, tempRekon(iRow,iCol-nFitSmplsL:iCol+nFitSmplsR), 'o');
            hold on
            sp22 = plot((iCol-nFitSmplsL:0.1:iCol+nFitSmplsR)*Settings.pitch, tempGaus((iCol-nFitSmplsL:0.1:iCol+nFitSmplsR)*Settings.pitch));
            hold off
            ylabel('[a.u]');
            xlabel('y [mm]');
            legend('data', 'fitted curve', 'Location', 'best'); 
        else
            set(sp1, 'CData', tempRekon);
            set(l1, 'YData' , [iRow iRow]*Settings.sampleDepth);
            set(sp21, 'YData', tempRekon(iRow,iCol-nFitSmplsL:iCol+nFitSmplsR));
            set(sp22, 'YData', tempGaus((iCol-nFitSmplsL:0.1:iCol+nFitSmplsR)*Settings.pitch));
        end
    end
    subplot(2,2,4)
    if iZ == 1
        sp4 = plot(abs(probeXs-probeXs(1)), maxAmp(iZ,:), '-o');
        xlabel('x [mm]');
        ylabel('[a.u]');
        legend('data', 'fitted curve', 'Location', 'best');
    else
        set(sp4, 'YData', maxAmp(iZ,:));
    end
end

%% show images
figure
imagesc(probeXsRltv, probeZsRltv,maxAmp);
xlabel('x [mm]');
ylabel('z [mm]');
% % figure
% % v = VideoWriter([fp 'calibrate acquired images'],'Uncompressed AVI');
% % v.FrameRate = 20;
% % open(v);
% for iX=1:nXs
%     for iZ=1:nZs
%         tempRekon = squeeze(rekon(iZ,iX,:,:));
%         imagesc(Settings.acqY, Settings.acqDepth, tempRekon);
%         xlabel('y [mm]');
%         ylabel('z [mm]');
%         colormap hot
%         colorbar
%         pause(0.15);
% %         frame = getframe(gcf);
% %         writeVideo(v,frame);
%     end
% end
% frame = getframe(gcf);
% writeVideo(v,frame);
% close(v);
% 
% %% save data
% a = fn{end,end};
% a = strsplit(a,'.rfe');
% save([fp a{1} '.mat'] , 'rekon', 'probeXs', 'probeZs', 'maxAmp');