function [fn,displacement,newFn] = process_fileName_3D(fn,fnEts,system,dispDir,dispStep)
% to process the file names in 3D reconstruction
% [fn,displacement,newFn] = process_fileName_3D(fn,system,dispDir)
% Input:
%   fn = file names
%   fnEts: file name extension. 3D projections will be saved under a
%   new name as fn_fnEts
%   system: the PAI system used to record data
%      1: MyLabOne with a no-laser probe
%      2: MyLabOne with a laser-integrated probe
%      3: Eagle
%   dispDir: displacement direction
%      1: axial (z) displacement
%      2: elevation (x) displacement
%   dispStep: displacement step size. Only needed when system = 2 or 3
% Output:
%   fn: sorted files
%   displacement: array containing the displacement
%   newFn: 3D projections will be saved under these new file names

%% check inputs
if(~ismember([1 2 3],system))
    error(['wrong input, ''system'' must be one of the following:\n'...
        ' 1: MyLabOne with a no-laser probe\n'...
        ' 2: MyLabOne with a laser-integrated probe\n'...
        ' 3: Eagle%s'],'');
    return;
end
if(~ismember([1 2],dispDir))
    error(['wrong input, ''dispDir'' must be one of the following:\n'...
        ' 1: axial (z) displacement\n%s'...
        ' 2: elevation (x) displacement'],'');
    return;
end

%% process file names
nFiles = length(fn);
if system==1 % myLabOne with a no-laser probe
    probeXs = zeros(nFiles,1);
    probeZs = zeros(nFiles,1);
    for iFile = 1:nFiles
        fnInfo = strsplit(fn{iFile}, '_');
        probeZs(iFile) = str2double(fnInfo{3});
        probeXs(iFile) = str2double(fnInfo{5});
    end
    xFileIds = probeZs==mode(probeZs);
    zFileIds = probeXs==mode(probeXs);
    commonX = probeXs(xFileIds&zFileIds);
    commonZ = probeZs(xFileIds&zFileIds);
    fnX = fn(xFileIds);
    fnZ = fn(zFileIds);
    probeXs = probeXs(xFileIds);
    probeZs = probeZs(zFileIds);
    if(dispDir==1) % z displacement
        [probeZs,idsZ] = sort(probeZs);
        if(abs(probeZs(1)-commonZ)>1e-3)
            [probeZs,idsZ2] = sort(probeZs,'descend');
            idsZ = idsZ(idsZ2);
        end
        displacement = abs(probeZs-probeZs(1));
        fn = fnZ(idsZ);
    elseif(dispDir==2) % x displacement
        [probeXs,idsX] = sort(probeXs);
        if(abs(probeXs(1)-commonX)>1e-3)
            [probeXs,idsX2] = sort(probeXs,'descend');
            idsX = idsX(idsX2);
        end
        displacement = abs(probeXs-probeXs(1));
        fn = fnX(idsX);
    end
    nFiles = length(fn);
    newFn = cell(nFiles, 1);
    for iFile=1:nFiles
        newFileNameInfo = strsplit(fn{iFile},'.rfe');
        newFn{iFile} = [newFileNameInfo{1} fnEts];
    end    
    
else % myLabOne or Eagle
    timeStamp = zeros(nFiles, 1);
    for iFile = 1:nFiles
        newFileNameInfo = strsplit(fn{iFile}, {'_', '.rfe'});
        timeStamp(iFile) = str2double(newFileNameInfo{3});
    end
    [~, sortOrd] = sort(timeStamp);
    fn = fn(sortOrd);
    nFiles = ceil(nFiles/2);
    if(dispDir==1) % z displacement
        fn = [fn(1); fn(end-nFiles+2:end)];
    elseif(dispDir==2) % x displacement
        fn = fn(1:nFiles);
    end    
    %%
    newFn = cell(nFiles, 1);
    for iFile = 1:nFiles
        newFileNameInfo = strsplit(fn{iFile}, '.rfe');
        newFn{iFile} = [newFileNameInfo{1}  fnEts];
    end
    displacement = (0:nFiles-1)*dispStep;
end